package exnihilocreatio.barrel.modes.compost;

import exnihilocreatio.barrel.IBarrelMode;
import exnihilocreatio.config.ModConfig;
import exnihilocreatio.networking.MessageBarrelModeUpdate;
import exnihilocreatio.networking.MessageCompostUpdate;
import exnihilocreatio.networking.PacketHandler;
import exnihilocreatio.registries.manager.ExNihiloRegistryManager;
import exnihilocreatio.registries.types.Compostable;
import exnihilocreatio.texturing.Color;
import exnihilocreatio.tiles.TileBarrel;
import exnihilocreatio.util.ItemInfo;
import exnihilocreatio.util.Util;
import lombok.Getter;
import lombok.Setter;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.FluidTank;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.items.ItemStackHandler;

import java.util.List;

public class BarrelModeCompost implements IBarrelMode {

    @Setter
    @Getter
    private float fillAmount = 0;
    @Setter
    private Color color = new Color("EEA96D");
    private Color whiteColor = new Color(1f, 1f, 1f, 1f);
    private Color originalColor;
    @Setter
    @Getter
    private float progress = 0;
    @Getter
    private IBlockState compostState;

    private BarrelItemHandlerCompost handler;

    public BarrelModeCompost() {
        handler = new BarrelItemHandlerCompost(null);
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean onBlockActivated(World world, TileBarrel barrel, BlockPos pos, IBlockState state, EntityPlayer player, EnumHand hand, EnumFacing side, float hitX, float hitY, float hitZ) {
        if (fillAmount == 0) {
            if (!player.func_184586_b(hand).func_190926_b()) {
                ItemInfo info = ItemInfo.getItemInfoFromStack(player.func_184586_b(hand));
                if (ExNihiloRegistryManager.COMPOST_REGISTRY.containsItem(info)) {
                    Compostable comp = ExNihiloRegistryManager.COMPOST_REGISTRY.getItem(info);
                    compostState = Block.func_149634_a(comp.getCompostBlock().getItem())
                            .getStateFromMeta(comp.getCompostBlock().getMeta());
                    PacketHandler.sendNBTUpdate(barrel);
                }
            }
        }
        if (fillAmount < 1 && compostState != null) {
            if (!player.func_184586_b(hand).func_190926_b()) {
                ItemInfo info = ItemInfo.getItemInfoFromStack(player.func_184586_b(hand));
                Compostable comp = ExNihiloRegistryManager.COMPOST_REGISTRY.getItem(info);
                IBlockState testState = Block.func_149634_a(comp.getCompostBlock().getItem())
                        .getStateFromMeta(comp.getCompostBlock().getMeta());

                if (ExNihiloRegistryManager.COMPOST_REGISTRY.containsItem(info) && compostState.equals(testState)) {
                    Compostable compost = ExNihiloRegistryManager.COMPOST_REGISTRY.getItem(info);

                    if (fillAmount == 0)
                        color = compost.getColor();
                    else
                        color = Color.average(color, compost.getColor(), compost.getValue());

                    fillAmount += compost.getValue();
                    if (fillAmount > 1)
                        fillAmount = 1;
                    if (!player.field_71075_bZ.field_75098_d)
                        player.func_184586_b(hand).func_190918_g(1);
                    PacketHandler.sendToAllAround(new MessageCompostUpdate(this.fillAmount, this.color, this.progress, barrel.func_174877_v()), barrel);
                    barrel.func_70296_d();
                    return true;
                }
            }
        } else if (progress >= 1) {
            Util.dropItemInWorld(barrel, player, new ItemStack(compostState == null ? Blocks.field_150350_a : compostState.func_177230_c(), 1, compostState == null ? 0 : compostState.func_177230_c().func_176201_c(compostState)), 0.02f);
            removeItem(barrel);
            return true;
        }

        return false;
    }

    public void removeItem(TileBarrel barrel) {
        progress = 0;
        fillAmount = 0;
        color = new Color("EEA96D");
        handler.setStackInSlot(0, ItemStack.field_190927_a);
        compostState = null;
        PacketHandler.sendToAllAround(new MessageCompostUpdate(this.fillAmount, this.color, this.progress, barrel.func_174877_v()), barrel);
        barrel.setMode("null");
        IBlockState state = barrel.func_145831_w().func_180495_p(barrel.func_174877_v());
        PacketHandler.sendToAllAround(new MessageBarrelModeUpdate("null", barrel.func_174877_v()), barrel);
        barrel.func_145831_w().func_175656_a(barrel.func_174877_v(), state);
    }

    @SuppressWarnings("deprecation")
    public boolean addItem(ItemStack stack, TileBarrel barrel) {
        if (fillAmount < 1) {
            if (stack != null) {
                ItemInfo info = ItemInfo.getItemInfoFromStack(stack);
                Compostable comp = ExNihiloRegistryManager.COMPOST_REGISTRY.getItem(info);
                IBlockState testState = Block.func_149634_a(comp.getCompostBlock().getItem())
                        .getStateFromMeta(comp.getCompostBlock().getMeta());

                if (ExNihiloRegistryManager.COMPOST_REGISTRY.containsItem(info) && compostState == null) {
                    compostState = testState;
                }

                if (ExNihiloRegistryManager.COMPOST_REGISTRY.containsItem(info) && compostState.equals(testState)) {
                    Compostable compost = ExNihiloRegistryManager.COMPOST_REGISTRY.getItem(info);

                    if (fillAmount == 0)
                        color = compost.getColor();
                    else
                        color = Color.average(color, compost.getColor(), compost.getValue());

                    fillAmount += compost.getValue();
                    if (fillAmount > 1)
                        fillAmount = 1;
                    PacketHandler.sendToAllAround(new MessageCompostUpdate(this.fillAmount, this.color, this.progress, barrel.func_174877_v()), barrel);
                    barrel.func_70296_d();
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public void update(TileBarrel barrel) {
        if (fillAmount >= 1 && progress < 1) {
            if (progress == 0) {
                originalColor = color;
            }

            progress += 1.0 / ModConfig.composting.ticksToFormDirt;

            color = Color.average(originalColor, whiteColor, progress);

            PacketHandler.sendToAllAround(new MessageCompostUpdate(this.fillAmount, this.color, this.progress, barrel.func_174877_v()), barrel);

            barrel.func_70296_d();
        }

        if (progress >= 1 && compostState != null) {
            barrel.setMode("block");
            PacketHandler.sendToAllAround(new MessageBarrelModeUpdate("block", barrel.func_174877_v()), barrel);

            barrel.getMode().addItem(new ItemStack(compostState.func_177230_c(), 1, compostState.func_177230_c().func_176201_c(compostState)), barrel);
        }
    }

    @Override
    public String getName() {
        return "compost";
    }

    @Override
    public List<String> getWailaTooltip(TileBarrel barrel, List<String> currenttip) {
        if (compostState != null)
            currenttip.add("Composting " + compostState.func_177230_c().func_149732_F());
        if (progress == 0) {
            currenttip.add(Math.round(fillAmount * 100) + "% full");
        } else {
            currenttip.add(Math.round(progress * 100) + "% complete");
        }
        return currenttip;
    }

    @Override
    public boolean isTriggerItemStack(ItemStack stack) {
        return ExNihiloRegistryManager.COMPOST_REGISTRY.containsItem(stack);
    }

    @Override
    public boolean isTriggerFluidStack(FluidStack stack) {
        return false;
    }

    @Override
    public void writeToNBT(NBTTagCompound tag) {
        tag.func_74776_a("fillAmount", fillAmount);
        tag.func_74768_a("color", color.toInt());
        if (originalColor != null)
            tag.func_74768_a("originalColor", originalColor.toInt());
        tag.func_74776_a("progress", progress);
        if (compostState != null) {
            tag.func_74778_a("block", Block.field_149771_c.func_177774_c(compostState.func_177230_c()).toString());
            tag.func_74768_a("meta", compostState.func_177230_c().func_176201_c(compostState));
        }

    }

    @SuppressWarnings("deprecation")
    @Override
    public void readFromNBT(NBTTagCompound tag) {
        fillAmount = tag.func_74760_g("fillAmount");
        this.color = new Color(tag.func_74762_e("color"));
        if (tag.func_74764_b("originalColor"))
            this.originalColor = new Color(tag.func_74762_e("originalColor"));
        this.progress = tag.func_74760_g("progress");
        if (tag.func_74764_b("block")) {
            Block block = Block.field_149771_c.func_82594_a(new ResourceLocation(tag.func_74779_i("block")));
            compostState = block.func_176203_a(tag.func_74762_e("meta"));
        }
    }

    @Override
    @SideOnly(Side.CLIENT)
    public TextureAtlasSprite getTextureForRender(TileBarrel barrel) {
        if (compostState == null)
            return Minecraft.func_71410_x().func_175602_ab().func_175023_a()
                    .func_178122_a(Blocks.field_150346_d.func_176223_P());
        return Minecraft.func_71410_x().func_175602_ab().func_175023_a()
                .func_178122_a(compostState);
    }

    @Override
    public float getFilledLevelForRender(TileBarrel barrel) {
        return fillAmount * 0.9375F;
    }

    @Override
    public Color getColorForRender() {
        return color;
    }

    @Override
    public ItemStackHandler getHandler(TileBarrel barrel) {
        handler.setBarrel(barrel);
        return handler;
    }

    @Override
    public FluidTank getFluidHandler(TileBarrel barrel) {
        return null;
    }

    @Override
    public boolean canFillWithFluid(TileBarrel barrel) {
        return false;
    }

}
